@@ -5,13 +5,13 @@ use crate::CliResult;
 use cached::proc_macro::cached;
 use censor::{Censor, Sex, Zealous};
 use chrono::{NaiveTime, Utc};
-use dateparser::parse_with;
 use dynfmt::Format;
 use eudex::Hash;
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use log::debug;
 use once_cell::sync::OnceCell;
 use qsv_currency::Currency;
+use qsv_dateparser::parse_with;
 use regex::Regex;
 use reverse_geocoder::{Locations, ReverseGeocoder};
 use serde::Deserialize;
@@ -424,7 +424,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         if args.flag_no_headers {
             return fail!("dynfmt operation requires headers.");
         } else {
-            safe_headers = util::safe_header_names(headers, false);
+            safe_headers = util::safe_header_names(&headers, false);
             let formatstr_re: &'static Regex = regex!(r"\{(?P<key>\w+)?\}");
             for format_fields in formatstr_re.captures_iter(&args.flag_formatstr) {
                 dynfmt_fields.push(format_fields.name("key").unwrap().as_str());
