@@ -62,16 +62,16 @@ Common options:
 ";
 
 #[derive(Clone, Deserialize)]
-struct Args {
-    arg_input: Option<String>,
-    flag_select: SelectColumns,
-    flag_limit: usize,
-    flag_asc: bool,
-    flag_no_nulls: bool,
-    flag_jobs: isize,
-    flag_output: Option<String>,
-    flag_no_headers: bool,
-    flag_delimiter: Option<Delimiter>,
+pub struct Args {
+    pub arg_input: Option<String>,
+    pub flag_select: SelectColumns,
+    pub flag_limit: usize,
+    pub flag_asc: bool,
+    pub flag_no_nulls: bool,
+    pub flag_jobs: usize,
+    pub flag_output: Option<String>,
+    pub flag_no_headers: bool,
+    pub flag_delimiter: Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -87,11 +87,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     wtr.write_record(vec!["field", "value", "count"])?;
     let head_ftables = headers.into_iter().zip(tables.into_iter());
     for (i, (header, ftab)) in head_ftables.enumerate() {
-        let mut header = header.to_vec();
-        if rconfig.no_headers {
-            header = (i + 1).to_string().into_bytes();
-        }
-        for (value, count) in args.counts(&ftab).into_iter() {
+        let header = if rconfig.no_headers {
+            (i + 1).to_string().into_bytes()
+        } else {
+            header.to_vec()
+        };
+        for (value, count) in args.counts(&ftab) {
             let count = count.to_string();
             let row = vec![&*header, &*value, count.as_bytes()];
             wtr.write_record(row)?;
@@ -106,7 +107,7 @@ type FTable = Frequencies<Vec<u8>>;
 type FTables = Vec<Frequencies<Vec<u8>>>;
 
 impl Args {
-    fn rconfig(&self) -> Config {
+    pub fn rconfig(&self) -> Config {
         Config::new(&self.arg_input)
             .delimiter(self.flag_delimiter)
             .no_headers(self.flag_no_headers)
@@ -134,13 +135,13 @@ impl Args {
             .collect()
     }
 
-    fn sequential_ftables(&self) -> CliResult<(Headers, FTables)> {
+    pub fn sequential_ftables(&self) -> CliResult<(Headers, FTables)> {
         let mut rdr = self.rconfig().reader()?;
         let (headers, sel) = self.sel_headers(&mut rdr)?;
         Ok((headers, self.ftables(&sel, rdr.byte_records())?))
     }
 
-    fn parallel_ftables(
+    pub fn parallel_ftables(
         &self,
         idx: &mut Indexed<fs::File, fs::File>,
     ) -> CliResult<(Headers, FTables)> {
@@ -200,12 +201,10 @@ impl Args {
     }
 
     fn njobs(&self) -> usize {
-        let num_cpus = util::num_cpus();
-        match self.flag_jobs {
-            0 => util::max_jobs(),
-            flag_jobs if flag_jobs < 0 => num_cpus,
-            flag_jobs if flag_jobs > num_cpus as isize => num_cpus,
-            _ => self.flag_jobs as usize,
+        if self.flag_jobs == 0 {
+            util::num_cpus()
+        } else {
+            self.flag_jobs
         }
     }
 }
