@@ -418,7 +418,7 @@ impl<R: io::Read + io::Seek> ValueIndex<R> {
             row_idx.write_u64::<BigEndian>(row.position().unwrap().byte())?;
 
             let fields: Vec<_> = sel.select(&row).map(|v| transform(v, casei)).collect();
-            if nulls || !fields.iter().any(|f| f.is_empty()) {
+            if nulls || !fields.iter().any(std::vec::Vec::is_empty) {
                 match val_idx.entry(fields) {
                     Entry::Vacant(v) => {
                         let mut rows = Vec::with_capacity(4);
@@ -453,9 +453,9 @@ impl<R> fmt::Debug for ValueIndex<R> {
             // This is just for debugging, so assume Unicode for now.
             let keys = keys
                 .iter()
-                .map(|k| String::from_utf8(k.to_vec()).unwrap())
+                .map(|k| String::from_utf8(k.clone()).unwrap())
                 .collect::<Vec<_>>();
-            writeln!(f, "({}) => {:?}", keys.join(", "), rows)?
+            writeln!(f, "({}) => {:?}", keys.join(", "), rows)?;
         }
         Ok(())
     }
@@ -469,15 +469,15 @@ fn transform(bs: &[u8], casei: bool) -> ByteString {
     match str::from_utf8(bs) {
         Err(_) => bs.to_vec(),
         Ok(s) => {
-            if !casei {
-                s.trim().as_bytes().to_vec()
-            } else {
+            if casei {
                 let norm: String = s
                     .trim()
                     .chars()
                     .map(|c| c.to_lowercase().next().unwrap())
                     .collect();
                 norm.into_bytes()
+            } else {
+                s.trim().as_bytes().to_vec()
             }
         }
     }
