@@ -314,7 +314,7 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
         flag_nulls: false,
         flag_nullcount: true,
         flag_dates: true,
-        flag_jobs: util::max_jobs() as isize,
+        flag_jobs: util::max_jobs(),
         flag_output: None,
         flag_no_headers: args.flag_no_headers,
         flag_delimiter: args.flag_delimiter,
@@ -403,7 +403,7 @@ fn get_unique_values(
         flag_limit: args.flag_enum_threshold,
         flag_asc: false,
         flag_no_nulls: true,
-        flag_jobs: util::max_jobs() as isize,
+        flag_jobs: util::max_jobs(),
         flag_output: None,
         flag_no_headers: args.flag_no_headers,
         flag_delimiter: args.flag_delimiter,
